@@ -673,6 +673,8 @@ class ALDocument(DADict):
 
         safe_key = space_to_underscore(key)
 
+        if not hasattr(self, "suffix_to_append"):
+            self.suffix_to_append = "preview"
         if append_matching_suffix and key == self.suffix_to_append:
             append_suffix: str = f"_{safe_key}"
         else:
@@ -793,7 +795,7 @@ class ALDocument(DADict):
     def safe_value(
         self,
         field_name: str,
-        overflow_message: str = None,
+        overflow_message: Optional[str] = None,
         preserve_newlines: bool = False,
         input_width: int = 80,
     ):
@@ -812,7 +814,7 @@ class ALDocument(DADict):
     def overflow_value(
         self,
         field_name: str,
-        overflow_message: str = None,
+        overflow_message: Optional[str] = None,
         preserve_newlines: bool = False,
         input_width: int = 80,
     ):
@@ -1186,7 +1188,7 @@ class ALDocumentBundle(DAList):
         view_icon: str = "eye",
         download_label: str = "Download",
         download_icon: str = "download",
-        zip_label: str = None,
+        zip_label: Optional[str] = None,
         zip_icon: str = "file-archive",
         append_matching_suffix: bool = True,
     ) -> str:
@@ -1544,7 +1546,7 @@ class ALExhibit(DAObject):
         pdfa: bool = False,
         add_page_numbers: bool = True,
         add_cover_page: bool = True,
-        filename: str = None,
+        filename: Optional[str] = None,
         append_matching_suffix: bool = True,
     ) -> DAFile:
         """
@@ -1596,6 +1598,7 @@ class ALExhibit(DAObject):
 class ALExhibitList(DAList):
     """
     Attributes:
+        maximum_size (int): the maximum size in bytes that the whole document is allowed to be
         auto_label (bool): Set to True if you want exhibits to be automatically numbered for purposes of cover page
                            and table of contents. Defaults to True.
         auto_labeler (Callable): (optional) a function or lambda to transform the index for each exhibit to a label.
@@ -1640,6 +1643,9 @@ class ALExhibitList(DAList):
         Returns:
             A DAfile containing the rendered exhibit list as a single file.
         """
+        if self.include_exhibit_cover_pages:
+            for exhibit in self:
+                exhibit.cover_page
         if self.include_table_of_contents and toc_pages != 1:
             self._update_page_numbers(toc_guess_pages=toc_pages)
         return pdf_concatenate(
@@ -1655,7 +1661,14 @@ class ALExhibitList(DAList):
             pdfa=pdfa,
         )
 
-    def _update_labels(self, auto_labeler: Callable = None) -> None:
+    def size_in_bytes(self):
+        """Gets the total size in bytes of each of the exhibit documents."""
+        full_size = 0
+        for exhibit in self.complete_elements():
+            full_size += sum((a_page.size_in_bytes() for a_page in exhibit.pages))
+        return full_size
+
+    def _update_labels(self, auto_labeler: Optional[Callable] = None) -> None:
         """
         Private method to refresh labels on all exhibits.
         Args:
@@ -1761,6 +1774,8 @@ class ALExhibitDocument(ALDocument):
         else:
             self.include_exhibit_cover_pages = True
             self.exhibits.include_exhibit_cover_pages = True
+        if hasattr(self, "maximum_size"):
+            self.exhibits.maximum_size = self.maximum_size
         if hasattr(self, "include_table_of_contents"):
             self.exhibits.include_table_of_contents = self.include_table_of_contents
         else:
