@@ -1,4 +1,4 @@
-from typing import Dict, List, Union
+from typing import Dict, List, Union, Optional
 from docassemble.base.util import (
     Address,
     Individual,
@@ -11,6 +11,7 @@ from docassemble.base.util import (
     url_action,
     get_config,
     phone_number_is_valid,
+    phone_number_formatted,
     validation_error,
     DAWeb,
     get_config,
@@ -71,24 +72,62 @@ class ALAddress(Address):
 
     def address_fields(
         self,
-        country_code: str = None,
-        default_state: str = None,
+        country_code: Optional[str] = None,
+        default_state: Optional[str] = None,
         show_country: bool = False,
-        show_if: Union[str, Dict[str, str]] = None,
+        show_county: bool = False,
+        show_if: Union[str, Dict[str, str], None] = None,
+        allow_no_address: bool = False,
     ):
-        fields = [
-            {
-                "label": str(self.address_label),
-                "address autocomplete": True,
-                "field": self.attr_name("address"),
-            },
-            {
-                "label": str(self.unit_label),
-                "field": self.attr_name("unit"),
-                "required": False,
-            },
-            {"label": str(self.city_label), "field": self.attr_name("city")},
-        ]
+        """
+        Return a YAML structure representing the list of fields for the object's address.
+        Optionally, allow the user to specify they do not have an address.
+        NOTE: if you set allow_no_address to True, you must make sure to trigger
+        the question with `users[0].address.has_no_address` rather than
+        `users[0].address.address`.
+        Optionally, add a `show if` modifier to each field. The `show if` modifier
+        will not be used if the `allow_no_address` modifier is used.
+        """
+        if allow_no_address:
+            fields = [
+                {
+                    "label": str(self.has_no_address_label),
+                    "field": self.attr_name("has_no_address"),
+                    "datatype": "yesno",
+                },
+                {
+                    "label": str(self.has_no_address_explanation_label),
+                    "field": self.attr_name("has_no_address_explanation"),
+                    "datatype": "area",
+                    "rows": 2,
+                    "help": str(self.has_no_address_explanation_help),
+                    "show if": self.attr_name("has_no_address"),
+                    "required": False,
+                },
+            ]
+        else:
+            fields = []
+        fields.extend(
+            [
+                {
+                    "label": str(self.address_label),
+                    "address autocomplete": bool(
+                        (get_config("google") or {}).get("google maps api key")
+                    ),
+                    "field": self.attr_name("address"),
+                },
+                {
+                    "label": str(self.unit_label),
+                    "field": self.attr_name("unit"),
+                    "required": False,
+                },
+            ]
+        )
+        if allow_no_address:
+            fields[-1]["hide if"] = self.attr_name("has_no_address")
+            fields[-2]["hide if"] = self.attr_name("has_no_address")
+
+        fields.append({"label": str(self.city_label), "field": self.attr_name("city")})
         if country_code:
             fields.append(
                 {
@@ -123,11 +162,21 @@ class ALAddress(Address):
                     "required": False,
                 }
             )
+        if allow_no_address:
+            fields[-1]["hide if"] = self.attr_name("has_no_address")
 
+        if show_county:
+            fields.append(
+                {
+                    "label": str(self.county_label),
+                    "field": self.attr_name("county"),
+                    "required": False,
+                }
+            )
         if show_country:
             fields.append(
                 {
-                    "label": self.country_label,
+                    "label": str(self.country_label),
                     "field": self.attr_name("country"),
                     "required": False,
                     "code": "countries_list()",
@@ -135,9 +184,11 @@ class ALAddress(Address):
                 }
             )
             # NOTE: using , "datatype": "combobox" might be nice but does not play together well w/ address autocomplete
-        if show_if:
-            for field in fields:
-                field["show if"] = show_if
+        if not allow_no_address:
+            # show if isn't compatible with the hide if logic for `allow_no_address`
+            if show_if:
+                for field in fields:
+                    field["show if"] = show_if
         return fields
 
     def formatted_unit(self, language=None, require=False, bare=False):
@@ -167,6 +218,19 @@ class ALAddress(Address):
             line_breaker = '</w:t><w:br/><w:t xml:space="preserve">'
         else:
             line_breaker = " [NEWLINE] "
+
+        if (
+            hasattr(self, "has_no_address")
+            and self.has_no_address
+            and hasattr(self, "has_no_address_explanation")
+        ):
+            return (
+                self.has_no_address_explanation
+                + line_breaker
+                + self.city
+                + line_breaker
+                + self.state
+            )
         if international:
             i18n_address = {}
             if (
@@ -231,6 +295,12 @@ class ALAddress(Address):
     def line_one(self, language=None, bare=False):
         """Returns the first line of the address, including the unit
         number if there is one."""
+        if (
+            hasattr(self, "has_no_address")
+            and self.has_no_address
+            and hasattr(self, "has_no_address_explanation")
+        ):
+            return self.has_no_address_explanation
         if self.city_only:
             return ""
         if (
@@ -255,6 +325,12 @@ class ALAddress(Address):
         bare=False,
     ):
         """Returns a one-line address.  Primarily used internally for geocoding."""
+        if (
+            hasattr(self, "has_no_address")
+            and self.has_no_address
+            and hasattr(self, "has_no_address_explanation")
+        ):
+            return f"{self.has_no_address_explanation}, {self.city} {self.state}"
         output = ""
         if self.city_only is False:
             if (
@@ -269,7 +345,8 @@ class ALAddress(Address):
                 the_unit = self.formatted_unit(language=language, bare=bare)
                 if the_unit != "":
                     output += ", " + the_unit
-            output += ", "
+            if output != "":
+                output += ", "
         # if hasattr(self, 'sublocality') and self.sublocality:
         #    output += str(self.sublocality) + ", "
         if hasattr(self, "sublocality_level_1") and self.sublocality_level_1:
@@ -296,6 +373,22 @@ class ALAddress(Address):
             output += ", " + country_name(self._get_country())
         return output
 
+    def normalized_address(self) -> Union[Address, "ALAddress"]:
+        """
+        Try geocoding the address, and if it succeeds, return the "long" normalized version of
+        the address. All methods are still available, such as my_address.normalized_address().block(), etc.,
+        but note that this will be a standard Address object, not an ALAddress object.
+
+        If geocoding fails, return the version of the address as entered by the user instead.
+        """
+        try:
+            self.geocode()
+        except:
+            pass
+        if self.was_geocoded_successfully() and hasattr(self, "norm_long"):
+            return self.norm_long
+        return self
+
 
 class ALAddressList(DAList):
     """Store a list of Address objects"""
@@ -388,12 +481,30 @@ class ALIndividual(Individual):
         else:
             return ""
 
-    def phone_numbers(self) -> str:
+    def phone_numbers(self, country=None) -> str:
         nums = []
         if hasattr(self, "mobile_number") and self.mobile_number:
-            nums.append({self.mobile_number: "cell"})
+            try:
+                nums.append(
+                    {
+                        phone_number_formatted(
+                            self.mobile_number, country=country
+                        ): "cell"
+                    }
+                )
+            except:
+                nums.append({self.mobile_number: "cell"})
         if hasattr(self, "phone_number") and self.phone_number:
-            nums.append({self.phone_number: "other"})
+            try:
+                nums.append(
+                    {
+                        phone_number_formatted(
+                            self.phone_number, country=country
+                        ): "other"
+                    }
+                )
+            except:
+                nums.append({self.phone_number: "other"})
         if len(nums) > 1:
             return comma_list(
                 [
@@ -449,12 +560,15 @@ class ALIndividual(Individual):
             return "%d weeks" % (int(dd.weeks),)
         return "%d days" % (int(dd.days),)
 
+    def normalized_address(self) -> Union[Address, ALAddress]:
+        return self.address.normalized_address()
+
     # This design helps us translate the prompts for common fields just once
     def name_fields(
         self,
         person_or_business: str = "person",
         show_suffix: bool = True,
-        show_if: Union[str, Dict[str, str]] = None,
+        show_if: Union[str, Dict[str, str], None] = None,
     ) -> List[Dict[str, str]]:
         """
         Return suitable field prompts for a name. If `person_or_business` is None, adds the
@@ -567,9 +681,11 @@ class ALIndividual(Individual):
     def address_fields(
         self,
         country_code: str = "US",
-        default_state: str = None,
+        default_state: Optional[str] = None,
         show_country: bool = False,
-        show_if: Union[str, Dict[str, str]] = None,
+        show_county: bool = False,
+        show_if: Union[str, Dict[str, str], None] = None,
+        allow_no_address: bool = False,
     ) -> List[Dict[str, str]]:
         """
         Return field prompts for address.
@@ -580,11 +696,13 @@ class ALIndividual(Individual):
             country_code=country_code,
             default_state=default_state,
             show_country=show_country,
+            show_county=show_county,
             show_if=show_if,
+            allow_no_address=allow_no_address,
         )
 
     def gender_fields(
-        self, show_help=False, show_if: Union[str, Dict[str, str]] = None
+        self, show_help=False, show_if: Union[str, Dict[str, str], None] = None
     ):
         """
         Return a standard gender input with "self described" option.
@@ -620,9 +738,9 @@ class ALIndividual(Individual):
 
     def language_fields(
         self,
-        choices: List[Dict[str, str]] = None,
+        choices: Optional[List[Dict[str, str]]] = None,
         style: str = "radio",
-        show_if: Union[str, Dict[str, str]] = None,
+        show_if: Union[str, Dict[str, str], None] = None,
     ):
         """Return a standard language picker with an "other" input. Language should be specified as ISO 639-2 or -3 codes so it is compatible with the language_name() function."""
         if not choices:
@@ -711,7 +829,7 @@ class ALIndividual(Individual):
     @property
     def initials(self):
         """Return the individual's initials, like QKS for Quinten K Steenhuis"""
-        return f"{self.name.first[:1]}{self.name.middle[:1] if hasattr(self.name,'middle') else ''}{self.name.last[:1]}"
+        return f"{self.name.first[:1]}{self.name.middle[:1] if hasattr(self.name,'middle') else ''}{self.name.last[:1] if hasattr(self.name, 'last') else ''}"
 
     def address_block(
         self, language=None, international=False, show_country=False, bare=False
@@ -834,7 +952,9 @@ def filter_letters(letter_strings: Union[List[str], str]) -> str:
 # Note: removed "combined_locations" because it is too tightly coupled to MACourts.py right now
 
 
-def fa_icon(icon: str, color: str = "primary", color_css: str = None, size: str = "sm"):
+def fa_icon(
+    icon: str, color: str = "primary", color_css: Optional[str] = None, size: str = "sm"
+):
     """
     Return HTML for a font-awesome icon of the specified size and color. You can reference
     a CSS variable (such as Bootstrap theme color) or a true CSS color reference, such as 'blue' or
